import argparse
import pathlib
import logging

from . import Config, NamingScheme, standardize_java_text

logger = logging.getLogger(__name__)


def test_files(repo: pathlib.Path) -> list[pathlib.Path]:
    return list(repo.rglob('*.java'))


def standardize_in_file(
    path: pathlib.Path,
    inline: bool,
    config: Config,
):
    with open(path) as f:
        text = f.read()

    text_updated = standardize_java_text(
        text,
        config,
    )
    if text_updated == text:
        return False

    if inline:
        with open(path, 'w') as f:
            f.write(text_updated)
    else:
        print(text_updated)
    return True


def argument_parser():
    argparser = argparse.ArgumentParser()
    argparser.add_argument('repo', type=pathlib.Path, help='Repository')
    argparser.add_argument('-i', action='store_true', help='Write to files')
    argparser.add_argument(
        '--javadoc',
        action='store_true',
        help='Generate Javadoc for tests',
    )
    argparser.add_argument(
        '--displayname',
        action='store_true',
        help='Generate DisplayName for tests',
    )
    argparser.add_argument(
        '--only-javadoc',
        action='store_true',
        help='Only change tests with an existing Javadoc',
    )
    argparser.add_argument(
        '--only-displayname',
        action='store_true',
        help='Only change tests with an existing DisplayName',
    )
    argparser.add_argument(
        '--only-todo',
        action='store_true',
        help='Only change tests where the test name contains the word "todo"',
    )
    argparser.add_argument(
        '--naming',
        type=NamingScheme,
        choices=list(NamingScheme),
        default=NamingScheme.FROM_DESC,
        help='How to derive the name of the test method',
    )
    return argparser


def main():
    logging.basicConfig(level='INFO')
    args = argument_parser().parse_args()

    config = Config(
        with_javadoc=args.javadoc,
        with_display_name=args.displayname,
        only_for_javadoc=args.only_javadoc,
        only_for_display_name=args.only_displayname,
        only_for_todo=args.only_todo,
        naming_scheme=args.naming,
    )

    num_modified = 0
    for path in test_files(args.repo):
        was_modified = standardize_in_file(
            path,
            inline=args.i,
            config=config,
        )
        if was_modified:
            num_modified += 1
            logger.info('Standardized %s', path)

    logger.info('Standardized %d files', num_modified)
    if not args.i:
        logger.warning('This was a dry run')
        logger.warning('Use -i to replace in files!')


if __name__ == '__main__':
    main()
